/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 3
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

struct VS_DRAWCOMMON
{
	float4 Position: POSITION0;
	float4 Color : COLOR0;
	float2 TexCoord: TEXCOORD0;
};

struct PS_DRAWTEXTURED
{
	float4 Position: POSITION0;
	float4 Color : COLOR0;
	float2 TexCoord: TEXCOORD0;
};

struct VS_DRAWPOSTPROC
{
	float4 Position: POSITION0;
	float2 TexCoord0: TEXCOORD0;
	float2 TexCoord1: TEXCOORD1;
};

struct PS_DRAWCOLOURED
{
	float4 Position: POSITION0;
	float4 Color : COLOR0;
};


#ifdef VERTEXSHADER
#include "fxVSIncludes.fx"

PS_DRAWTEXTURED DrawTexturedVS (VS_DRAWCOMMON vs_in)
{
	PS_DRAWTEXTURED vs_out;

	vs_out.Position = GetPosition (OrthoMatrix, vs_in.Position);
	vs_out.Color = vs_in.Color;
	vs_out.TexCoord = vs_in.TexCoord;

	return vs_out;
}

VS_DRAWPOSTPROC PostProcessVS (VS_DRAWPOSTPROC vs_in)
{
	VS_DRAWPOSTPROC vs_out;

	vs_out.Position = GetPosition (identityMatrix, vs_in.Position);
	vs_out.TexCoord0 = vs_in.TexCoord0;
	vs_out.TexCoord1 = vs_in.TexCoord1;

	return vs_out;
}

PS_DRAWCOLOURED DrawColouredVS (VS_DRAWCOMMON vs_in)
{
	PS_DRAWCOLOURED vs_out;

	vs_out.Position = GetPosition (identityMatrix, vs_in.Position);
	vs_out.Color = vs_in.Color;

	return vs_out;
}

PS_DRAWCOLOURED DrawPolyBlendVS (VS_DRAWCOMMON vs_in)
{
	PS_DRAWCOLOURED vs_out;

	vs_out.Position = GetPosition (identityMatrix, vs_in.Position);
	vs_out.Color = vs_in.Color;

	return vs_out;
}
#endif


#ifdef PIXELSHADER
#include "fxPSIncludes.fx"

#define drawSampler tmu0Sampler
#define screenSampler tmu3Sampler
#define postSampler tmu4Sampler

float4 DrawTexturedPS (PS_DRAWTEXTURED ps_in) : COLOR0
{
	return ApplyGammaPS (tex2D (drawSampler, ps_in.TexCoord), 1.0f) * ps_in.Color;
}

float4 DrawUnderwaterPS (VS_DRAWPOSTPROC ps_in) : COLOR0
{
	// run the underwater warp, merging it with the polyblend for added perf
	// scroll the warp in opposite dirs so that it doesn't cycle so obviously
	// offsetting the timescale slightly too to help with the effect
	float2 warp = float2 (
		// fixme - move warptime to VS
		tex2D (postSampler, ps_in.TexCoord1 + WarpTime.x).r,
		tex2D (postSampler, ps_in.TexCoord1 - WarpTime.y).g
	) * (1.0f / 128.0f);

	// don't apply gamma because it's just reading back an already-corrected scene
	return lerp (tex2D (screenSampler, ps_in.TexCoord0 + warp), ApplyGammaPS (PolyBlend, 1.0f), PolyBlend.a);
}

float4 DrawColouredPS (PS_DRAWCOLOURED ps_in) : COLOR0
{
	return ApplyGammaPS (ps_in.Color, 1.0f);
}

float4 DrawPolyBlendPS (PS_DRAWCOLOURED ps_in) : COLOR0
{
	return ApplyGammaPS (PolyBlend, 1.0f);
}
#endif

