/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "gl_local.h"


/*
=================
R_DrawBrushModel
=================
*/
void R_DrawBrushModel (entity_t *e)
{
	vec3_t bbox[8];

	// no surfaces
	if (e->model->nummodelsurfaces == 0) return;

	// culled
	if (R_CullEntityForBBox (e, bbox, e->model->mins, e->model->maxs)) return;

	R_InverseTransform (&e->localmatrix, e->modelorg, r_newrefdef.vieworigin);
	R_NewImageFrame ();
	R_ClearTextureChains ();

	msurface_t *surf = &e->model->surfaces[e->model->firstmodelsurface];

	// draw texture - !!!!! no state should be set before now as all surfs may be added to the alpha lists !!!!!
	for (int i = 0; i < e->model->nummodelsurfaces; i++, surf++)
	{
		// find which side of the node we are on
		cplane_t *plane = surf->plane;
		float dot = Vector3Dot (e->modelorg, plane->normal) - plane->dist;
		int sidebit = (dot >= 0) ? 0 : SURF_PLANEBACK;

		if ((surf->flags & SURF_PLANEBACK) != sidebit) continue;

		// draw the polygon
		R_ChainSurface (e, surf);
	}

	R_DrawTextureChains (e);

	if (r_showbboxes->value) R_DrawBBoxBox (bbox);
}


