/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// r_main.c

#include "gl_local.h"


viddef_t	vid;
refimport_t	*ri;
model_t		*r_worldmodel;

glconfig_t gl_config;
glstate_t  gl_state;

image_t		*r_notexture;		// use for bad textures
image_t		*r_blacktexture;
image_t		*r_greytexture;
image_t		*r_whitetexture;

cplane_t	frustum[4];

int			r_visframecount;	// bumped when going to a new PVS
int			r_framecount;		// used for dlight push checking

int			c_brush_polys, c_alias_polys, c_draw_call;

glmatrix	r_view_matrix;
glmatrix	r_projection_matrix;
glmatrix	r_mvp_matrix;


// screen size info
refdef_t	r_newrefdef;

mleaf_t *r_viewleaf[2];

cvar_t	*r_norefresh;
cvar_t	*r_drawentities;
cvar_t	*r_drawworld;
cvar_t	*r_speeds;
cvar_t	*r_novis;
cvar_t	*r_nocull;
cvar_t	*r_lerpmodels;
cvar_t	*r_lefthand;
cvar_t	*r_lightlevel;	// FIXME: This is a HACK to get the client's light level
cvar_t	*r_showbboxes;
cvar_t	*r_shownullmodels;
cvar_t	*r_fullbright;

cvar_t	*gl_dynamic;
cvar_t	*gl_modulate;
cvar_t	*vid_gamma;
cvar_t	*vid_brightness;
cvar_t	*intensity;
cvar_t	*gl_clear;
cvar_t	*gl_cull;
cvar_t	*gl_polyblend;
cvar_t	*gl_texturemode;
cvar_t	*gl_textureanisotropy;
cvar_t	*gl_lockpvs;
cvar_t	*gl_lockfrustum;


void Main_OnCreate (void) {}
void Main_OnDestroy (void) {}

void Main_OnLost (void)
{
	SAFE_RELEASE (d3d_FinishEvent);
}

void Main_OnReset (void)
{
	d3d_Device->lpVtbl->CreateQuery (d3d_Device, D3DQUERYTYPE_EVENT, &d3d_FinishEvent);
}

void Main_Register (void)
{
	static qboolean registered = false;

	if (!registered)
	{
		R_RegisterHandler (Main_OnCreate, Main_OnLost, Main_OnReset, Main_OnDestroy);
		registered = true;
	}
}


void R_Finish (void)
{
	if (d3d_FinishEvent)
	{
		d3d_FinishEvent->lpVtbl->Issue (d3d_FinishEvent, D3DISSUE_END);
		while (d3d_FinishEvent->lpVtbl->GetData (d3d_FinishEvent, NULL, 0, D3DGETDATA_FLUSH) == S_FALSE);
	}
}


/*
==================
R_ScreenShot_f
==================
*/
void R_ScreenShot_f (void)
{
	char checkname[MAX_PATH];

	// create the scrnshots directory if it doesn't exist
	Com_sprintf (checkname, sizeof (checkname), "%s/scrnshot", ri->FS_Gamedir ());
	Sys_Mkdir (checkname);

	// find a file name to save it to 
	for (int i = 0; i <= 99; i++)
	{
		sprintf (checkname, "%s/scrnshot/quake%02i.jpg", ri->FS_Gamedir (), i);

		if (!FS_FileExists (checkname))
		{
			IDirect3DSurface9 *surf = NULL;

			// sync the GPU
			R_Finish ();

			if (SUCCEEDED (d3d_Device->lpVtbl->GetRenderTarget (d3d_Device, 0, &surf)))
			{
				if (SUCCEEDED (D3DXSaveSurfaceToFile (checkname, D3DXIFF_JPG, surf, NULL, NULL)))
					ri->Con_Printf (PRINT_ALL, "Wrote quake%02i.jpg\n", i);

				surf->lpVtbl->Release (surf);
			}

			return;
		}
	}

	ri->Con_Printf (PRINT_ALL, "SCR_ScreenShot_f: Couldn't create a file\n");
}


/*
=================
R_CullBox

Returns true if the box is completely outside the frustom
=================
*/
qboolean R_CullBox (vec3_t mins, vec3_t maxs)
{
	if (r_nocull->value)
		return false;

	for (int i = 0; i < 4; i++)
	{
		if (BOX_ON_PLANE_SIDE (mins, maxs, &frustum[i]) == 2)
			return true;
	}

	return false;
}


int R_TestFrustumPoint (float *point)
{
	int mask = 0;

	for (int f = 0; f < 4; f++)
	{
		float dp = Vector3Dot (frustum[f].normal, point);

		if ((dp - frustum[f].dist) < 0)
			mask |= (1 << f);
	}

	return mask;
}


qboolean R_CullEntityForBBox (entity_t *e, vec3_t *bbox, vec3_t mins, vec3_t maxs)
{
	if (e->rotated)
	{
		// full matrix transform
		for (int i = 0; i < 8; i++)
		{
			vec3_t tmp;

			tmp[0] = (i & 1) ? mins[0] : maxs[0];
			tmp[1] = (i & 2) ? mins[1] : maxs[1];
			tmp[2] = (i & 4) ? mins[2] : maxs[2];

			// rotate the bounding box
			R_Transform (&e->localmatrix, bbox[i], tmp);
		}
	}
	else if (e->translated)
	{
		// just move the point
		for (int i = 0; i < 8; i++)
		{
			bbox[i][0] = ((i & 1) ? mins[0] : maxs[0]) + e->currorigin[0];
			bbox[i][1] = ((i & 2) ? mins[1] : maxs[1]) + e->currorigin[1];
			bbox[i][2] = ((i & 4) ? mins[2] : maxs[2]) + e->currorigin[2];
		}
	}
	else
	{
		// not transformed at all
		for (int i = 0; i < 8; i++)
		{
			bbox[i][0] = (i & 1) ? mins[0] : maxs[0];
			bbox[i][1] = (i & 2) ? mins[1] : maxs[1];
			bbox[i][2] = (i & 4) ? mins[2] : maxs[2];
		}
	}

	// and now do the culling test
	int aggregatemask = ~0;

	for (int i = 0; i < 8; i++)
	{
		int mask = 0;

		for (int f = 0; f < 4; f++)
			if ((Vector3Dot (frustum[f].normal, bbox[i]) - frustum[f].dist) < 0)
				mask |= (1 << f);

		aggregatemask &= mask;
	}

	if (aggregatemask)
		return true;
	else return false;
}


void R_DrawSingleEntityOnList (entity_t *e)
{
	if (e->flags & RF_BEAM)
		R_DrawBeam (e);
	else if (!e->model)
		R_DrawNullModel (e);
	else
	{
		switch (e->model->type)
		{
		case mod_alias:
			R_DrawAliasModel (e);
			break;

		case mod_brush:
			R_DrawBrushModel (e);
			break;

		case mod_sprite:
			R_DrawSpriteModel (e);
			break;

		default:
			ri->Sys_Error (ERR_DROP, "Bad modeltype");
			break;
		}
	}
}


__inline qboolean R_IsEntityTranslated (float *origin) { return (origin[0] || origin[1] || origin[2]); }
__inline qboolean R_IsEntityRotated (float *angles) { return (angles[0] || angles[1] || angles[2]); }


/*
=============
R_DrawEntitiesOnList
=============
*/
void R_DrawEntitiesOnList (refdef_t *rd)
{
	// nothing to draw...
	if (!r_drawentities->value) return;
	if (!rd->num_entities) return;

	// prepare entity transforms
	for (int i = 0; i < rd->num_entities; i++)
	{
		entity_t *e = &rd->entities[i];

		// ensure that the local matrix is initially at identity for all entity types
		R_IdentityMatrix (&e->localmatrix);

		// make surface distance and cullbox calcs faster
		e->rotated = R_IsEntityRotated (e->angles);
		e->translated = R_IsEntityTranslated (e->currorigin);

		if (e->flags & RF_BEAM)
		{
			R_AddAlphaObject (e, NULL, e->alpha);
			continue;
		}

		// allow testing with NULL models in the scene
		if (!e->model || r_shownullmodels->value)
		{
			if (e->translated) R_TranslateMatrix (&e->localmatrix, e->currorigin[0], e->currorigin[1], e->currorigin[2]);
			if (e->rotated) R_RotateMatrix (&e->localmatrix, -e->angles[0], e->angles[1], -e->angles[2]);
			R_DrawNullModel (e);
		}
		else
		{
			if (e->model->type == mod_brush)
			{
				if (e->translated) R_TranslateMatrix (&e->localmatrix, e->currorigin[0], e->currorigin[1], e->currorigin[2]);
				if (e->rotated) R_RotateMatrix (&e->localmatrix, e->angles[0], e->angles[1], e->angles[2]);
			}
			else if (e->model->type == mod_alias)
			{
				if (e->translated) R_TranslateMatrix (&e->localmatrix, e->currorigin[0], e->currorigin[1], e->currorigin[2]);
				if (e->rotated) R_RotateMatrix (&e->localmatrix, e->angles[0], e->angles[1], -e->angles[2]);
			}

			// handle the entity appropriately
			// bmodels must add their alpha surfaces individually, other model types may be drawn as a whole
			// non-alpha entities may be drawn immediately
			if (e->model->type == mod_brush)
				R_DrawSingleEntityOnList (e);
			else if (e->flags & RF_TRANSLUCENT)
				R_AddAlphaObject (e, NULL, e->alpha);
			else R_DrawSingleEntityOnList (e);
		}
	}
}


//=======================================================================

int SignbitsForPlane (cplane_t *out)
{
	// for fast box on planeside test
	int bits = 0;

	for (int j = 0; j < 3; j++)
	{
		if (out->normal[j] < 0)
			bits |= 1 << j;
	}

	return bits;
}


void R_SetFrustum (refdef_t *rd)
{
	if (gl_lockfrustum->value) return;

	// extract the frustum from the MVP matrix
	frustum[0].normal[0] = r_mvp_matrix.m4x4[0][3] - r_mvp_matrix.m4x4[0][0];
	frustum[0].normal[1] = r_mvp_matrix.m4x4[1][3] - r_mvp_matrix.m4x4[1][0];
	frustum[0].normal[2] = r_mvp_matrix.m4x4[2][3] - r_mvp_matrix.m4x4[2][0];

	frustum[1].normal[0] = r_mvp_matrix.m4x4[0][3] + r_mvp_matrix.m4x4[0][0];
	frustum[1].normal[1] = r_mvp_matrix.m4x4[1][3] + r_mvp_matrix.m4x4[1][0];
	frustum[1].normal[2] = r_mvp_matrix.m4x4[2][3] + r_mvp_matrix.m4x4[2][0];

	frustum[2].normal[0] = r_mvp_matrix.m4x4[0][3] + r_mvp_matrix.m4x4[0][1];
	frustum[2].normal[1] = r_mvp_matrix.m4x4[1][3] + r_mvp_matrix.m4x4[1][1];
	frustum[2].normal[2] = r_mvp_matrix.m4x4[2][3] + r_mvp_matrix.m4x4[2][1];

	frustum[3].normal[0] = r_mvp_matrix.m4x4[0][3] - r_mvp_matrix.m4x4[0][1];
	frustum[3].normal[1] = r_mvp_matrix.m4x4[1][3] - r_mvp_matrix.m4x4[1][1];
	frustum[3].normal[2] = r_mvp_matrix.m4x4[2][3] - r_mvp_matrix.m4x4[2][1];

	for (int i = 0; i < 4; i++)
	{
		Vector3Normalize (frustum[i].normal);
		frustum[i].type = PLANE_ANYZ;
		frustum[i].dist = Vector3Dot (rd->vieworigin, frustum[i].normal);
		frustum[i].signbits = SignbitsForPlane (&frustum[i]);
	}
}

//=======================================================================

/*
===============
R_SetupFrame
===============
*/
void R_SetupFrame (refdef_t *rd)
{
	r_framecount++;

	// allow them to scale the tint
	if (gl_polyblend->value > 0)
		rd->blend[3] *= gl_polyblend->value;
	else rd->blend[3] = 0;

	// current viewcluster
	if (!(rd->rdflags & RDF_NOWORLDMODEL))
	{
		r_viewleaf[0] = Mod_PointInLeaf (rd->vieworigin, r_worldmodel);

		if (r_viewleaf[0]->contents & CONTENTS_SOLID)
		{
			// clear the view to our cshift colour
			byte bgra[4];

			if (rd->blend[0] < 0) bgra[2] = 0; else if (rd->blend[0] > 1) bgra[2] = 255; else bgra[2] = rd->blend[0] * 255;
			if (rd->blend[1] < 0) bgra[1] = 0; else if (rd->blend[1] > 1) bgra[1] = 255; else bgra[1] = rd->blend[1] * 255;
			if (rd->blend[2] < 0) bgra[0] = 0; else if (rd->blend[2] > 1) bgra[0] = 255; else bgra[0] = rd->blend[2] * 255;
			if (rd->blend[3] < 0) bgra[3] = 0; else if (rd->blend[3] > 1) bgra[3] = 255; else bgra[3] = rd->blend[3] * 255;

			d3d_Device->lpVtbl->Clear (d3d_Device, 0, NULL, D3DCLEAR_TARGET, *((unsigned *) bgra), 1.0f, 0);
		}
	}

	c_brush_polys = 0;
	c_alias_polys = 0;
	c_draw_call = 0;
}


/*
=============
R_SetupGL
=============
*/
void R_SetupGL (refdef_t *rd)
{
	// no more "bottom-left is the origin" crap and no more scaling so we can just use the refdef directly
	D3DVIEWPORT9 vp = {rd->x, rd->y, rd->width, rd->height, 0, 1};
	R_SetViewport (&vp);

	R_IdentityMatrix (&r_projection_matrix);
	R_FrustumMatrix (&r_projection_matrix, rd->fov_x, rd->fov_y);

	// now setup the camera position into the view matrix
	R_IdentityMatrix (&r_view_matrix);
	R_CameraMatrix (&r_view_matrix, rd->vieworigin, rd->viewangles);

	// derive a full MVP (the initial "M" is going to be identity so we can ignore it for now)
	R_MultMatrix (&r_mvp_matrix, &r_view_matrix, &r_projection_matrix);

	// this now goes to our shaders in register 0
	R_SetVertexShaderConstant4fv (VSC_WORLDMATRIX, r_mvp_matrix.m16, 4);

	// take the view vectors from the view matrix - these are the only places they're used
	R_SetVertexShaderConstant3f (VSC_VUP, r_view_matrix.m4x4[0][1], r_view_matrix.m4x4[1][1], r_view_matrix.m4x4[2][1]);
	R_SetVertexShaderConstant3f (VSC_VPN, -r_view_matrix.m4x4[0][2], -r_view_matrix.m4x4[1][2], -r_view_matrix.m4x4[2][2]);
	R_SetVertexShaderConstant3f (VSC_VRIGHT, r_view_matrix.m4x4[0][0], r_view_matrix.m4x4[1][0], r_view_matrix.m4x4[2][0]);
	R_SetVertexShaderConstant4fv (VSC_RORIGIN, rd->vieworigin, 1);

	// setup transforms for warp surfaces once only
	R_SetupWaterTransform (rd);
	R_SetupSkyTransform (rd);

	if (rd->rdflags & RDF_NOWORLDMODEL)
	{
		// clear out the portion of the screen that the NOWORLDMODEL defines (no more "bottom-left is the origin" crap)
		// is clear constrained to the current viewport?  i.e do we need to do this with a rect at all?
		// the old clear colour looked bad as it jutted against the textbox borders in an ugly manner; this is better.
		// add D3DCLEAR_TARGET if we want to test positioning
		D3DRECT sr = {rd->x, rd->y, rd->x + rd->width, rd->y + rd->height};
		d3d_Device->lpVtbl->Clear (d3d_Device, 1, &sr, D3DCLEAR_ZBUFFER, 0xff444444, 1.0f, 0);
	}
}


/*
=============
R_Clear
=============
*/
void R_Clear (void)
{
	if (gl_lockpvs->value || gl_lockfrustum->value)
		d3d_Device->lpVtbl->Clear (d3d_Device, 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER | D3DCLEAR_STENCIL, 0xffff4c01, 1.0f, 0);
	else if (gl_clear->value)
		d3d_Device->lpVtbl->Clear (d3d_Device, 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER | D3DCLEAR_STENCIL, 0, 1.0f, 0);
	else d3d_Device->lpVtbl->Clear (d3d_Device, 0, NULL, D3DCLEAR_ZBUFFER | D3DCLEAR_STENCIL, 0, 1.0f, 0);
}


void R_Flash (refdef_t *rd) // AHH-AHH!!!
{
	if (gl_polyblend->value > 0 && rd->blend[3] > 0)
	{
		// draw it
		Draw_PolyBlend (rd->blend);

		// disable for the next frame
		rd->blend[3] = 0;
	}
}


/*
===================
R_RenderFrame

===================
*/
void R_RenderFrame (refdef_t *rd)
{
	// this is the world entity and it's declared here otherwise the current entity
	// will be stack garbage when drawing alpha surfaces from the world later on
	entity_t r_worldentity;

	if (r_norefresh->value)
		return;

	r_newrefdef = *rd;

	if (!r_worldmodel && !(rd->rdflags & RDF_NOWORLDMODEL))
		ri->Sys_Error (ERR_DROP, "R_RenderView: NULL worldmodel");

	c_brush_polys = 0;
	c_alias_polys = 0;
	c_draw_call = 0;

	R_SetupFrame (rd);

	R_SetupGL (rd);

	R_SetFrustum (rd);

	R_RunDLights (rd);

	R_MarkLeaves (rd);

	R_ClearAlphaObjects ();

	R_UpdateLightStyles (rd);

	// rationalization: entities will mostly occlude world geometry, so drawing entities *before* the world
	// allows us to get early-z advantages on the world itself.  this may also be friendly for merging
	// non-moving bmodels into the world draw.
	R_DrawEntitiesOnList (rd);

	R_DrawWorld (rd, &r_worldentity);

	R_DrawAlphaObjects ();

	R_DrawParticles (rd);

	// done before the polyblend so that we can disable it if it runs
	if (rd->rdflags & RDF_UNDERWATER)
	{
		// setup a post-processing pass for the underwater warp
		Draw_RunPostProcessStep (R_DrawUnderwaterWarp);

		// disable the polyblend (it's merged with the underwater warp for perf reasons)
		rd->blend[3] = 0;
	}
	else R_Flash (rd);	// AHH-AHH!!!

	if (r_speeds->value)
	{
		ri->Con_Printf (PRINT_ALL, "%6i wpoly %6i epoly %4i draw\n",
			c_brush_polys,
			c_alias_polys,
			c_draw_call);
	}

	R_SetLightLevel ();
}


/*
===================
R_BeginFrame
===================
*/
qboolean R_BeginFrame (void)
{
	if (!d3d_Device) return false;

	if (gl_state.DeviceLost)
	{
		// make another attempt at recovering a lost device
		R_HandleLostDevice ();
		return false;
	}

	// change modes if necessary
	if (R_ShouldModeChange (ri->vars))
	{
		// run a device reset and skip the next frame
		R_ResetDevice (ri->vars);
		return false;
	}

	// issue a beginscene call
	if (FAILED (d3d_Device->lpVtbl->BeginScene (d3d_Device)))
	{
		// this should never happen; try to recover the device, skip the next frame
		R_ResetDevice (ri->vars);
		return false;
	}

	// texturemode stuff
	if (gl_texturemode->modified || gl_textureanisotropy->modified)
	{
		R_TextureMode (gl_texturemode->string, (int) gl_textureanisotropy->value);
		gl_texturemode->modified = false;
		gl_textureanisotropy->modified = false;
	}

	// clear screen if desired
	R_Clear ();

	// set up gamma
	if (vid_gamma->value > 0.0f)
		R_SetPixelShaderConstant1f (PSC_GAMMA, vid_gamma->value);
	else R_SetPixelShaderConstant1f (PSC_GAMMA, 0);

	// set up intensity
	if (intensity->value > 0.0f)
		R_SetPixelShaderConstant1f (PSC_INTENSITY, intensity->value);
	else R_SetPixelShaderConstant1f (PSC_INTENSITY, 0);

	// set up brightness
	if (vid_brightness->value > 0.0f)
		R_SetPixelShaderConstant1f (PSC_BRIGHTNESS, vid_brightness->value);
	else R_SetPixelShaderConstant1f (PSC_BRIGHTNESS, 0);

	return true;
}


/*
===================
R_EndFrame

Responsible for doing a swapbuffers and possibly for other stuff
as yet to be determined.
===================
*/
void R_EndFrame (void)
{
	if (!d3d_Device) return;

	if (FAILED (d3d_Device->lpVtbl->EndScene (d3d_Device)))
	{
		// this should never happen; try to recover the device
		R_ResetDevice (ri->vars);
		return;
	}

	if ((d3d_Device->lpVtbl->Present (d3d_Device, NULL, NULL, NULL, NULL)) == D3DERR_DEVICELOST)
	{
		// the device is lost and make an initial attempt at recovering it, subsequent attempts will be made at the start of each frame
		gl_state.DeviceLost = true;
		R_HandleLostDevice ();
	}
}


