/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// gl_warp.c -- sky and water polygons

#include "gl_local.h"
#include "resource.h"

#define NOISETEXTURE_SIZE	16

IDirect3DVertexShader9 *d3d_SurfTurbVS = NULL;
IDirect3DPixelShader9 *d3d_SurfTurbPS = NULL;
IDirect3DPixelShader9 *d3d_DrawUnderwaterPS = NULL;

image_t d3d_NoiseTexture;

void Warp_OnCreate (void)
{
	// fix the random seed for the noise texture so that it's always the same
	srand (666);

	// and load our shaders
	R_CreateShaders (SurfTurbVS, SurfTurbPS, IDR_SURFSHADER);
	d3d_DrawUnderwaterPS = R_CreatePixelShader (IDR_DRAWSHADER, "DrawUnderwaterPS");

	d3d_Device->lpVtbl->CreateTexture (
		d3d_Device,
		NOISETEXTURE_SIZE,
		NOISETEXTURE_SIZE,
		1,
		0,
		D3DFMT_V8U8,
		D3DPOOL_MANAGED,
		&d3d_NoiseTexture.texture,
		NULL);

	D3DLOCKED_RECT lockrect;

	if (SUCCEEDED (d3d_NoiseTexture.texture->lpVtbl->LockRect (d3d_NoiseTexture.texture, 0, &lockrect, NULL, 0)))
	{
		char *vu = (char *) lockrect.pBits;

		// fixme - adjust for pitch
		for (int y = 0; y < NOISETEXTURE_SIZE; y++)
		{
			for (int x = 0; x < NOISETEXTURE_SIZE; x++)
			{
				*vu++ = (rand () & 255) - 128;
				*vu++ = (rand () & 255) - 128;
			}
		}

		d3d_NoiseTexture.texture->lpVtbl->UnlockRect (d3d_NoiseTexture.texture, 0);
	}
}


void Warp_OnReset (void) {}
void Warp_OnLost (void) {}


void Warp_OnDestroy (void)
{
	R_ReleaseTexture (&d3d_NoiseTexture);
}


void Warp_Register (void)
{
	static qboolean registered = false;

	if (!registered)
	{
		R_RegisterHandler (Warp_OnCreate, Warp_OnLost, Warp_OnReset, Warp_OnDestroy);
		registered = true;
	}
}


void R_DrawWaterChain (msurface_t *chain)
{
	surfbatch_t batch;

	// RenderState is set in the caller, not here, so that we can reuse it for alpha surfaces
	R_SetVertexShader (d3d_SurfTurbVS);
	R_SetPixelShader (d3d_SurfTurbPS);

	R_BeginBatchingSurfaces (&batch);

	// water surfs could be animated in the original and so we should too
	for (msurface_t *surf = chain; surf; surf = surf->texturechain)
		R_BatchSurface (&batch, surf, R_TextureAnimation (surf->texinfo, surf->animframe));

	R_EndBatchingSurfaces (&batch);
}


void R_SetupWaterTransform (refdef_t *rd)
{
	double junk = 0.0f;

	// cycle in increments of 2 * M_PI so that the sine warp will wrap correctly
	float rdt = modf ((double) rd->time / (M_PI * 2.0), &junk) * (M_PI * 2.0);

	R_SetVertexShaderConstant2f (VSC_WARPTIME, rdt, rdt);
	R_SetVertexShaderConstant2f (VSC_TEXSCROLL, modf ((double) rd->time / 40.0, &junk) * -64.0, 0);
}


void R_DrawUnderwaterWarp (void)
{
	double junk[2] = {0, 0};

	R_SetPixelShader (d3d_DrawUnderwaterPS);
	R_SetTexture (TS_POSTPROC, &d3d_NoiseTexture);

	R_SetPixelShaderConstant4fv (PSC_POLYBLEND, r_newrefdef.blend, 1);

	// fixme - move this to VS
	R_SetPixelShaderConstant2f (PSC_WARPTIME, modf (r_newrefdef.time * 0.09, &junk[0]), modf (r_newrefdef.time * 0.11, &junk[1]));
}


